/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "config.h"
#include "appwindow.h"
#include "appfilemenu.h"
#include "appbuttons.h"
#include "dialogs.h"
#include "i18n.h"
#include <gtk--/box.h>
#include <gtk--/menubar.h>
#include <stdio.h>

bool
AppWindow::reallyClose()
{
  QuestionDialog *question_dialog;
  bool answer = true;

  if (!orderedsets->areAllSetsSaved()) {
    question_dialog = new QuestionDialog(_("Really quit?"),
                                         _("Some files unsaved."),
                                         string(_("Quit the")) + " " + 
					 title + "?");
    answer = question_dialog->exec();
    delete question_dialog;
  }
  return answer;
}

gint
AppWindow::delete_event_impl(GdkEventAny* e)
{
  if (reallyClose()) return Gtk::Window::delete_event_impl(e);
  return true;
}

void
AppWindow::help()
{

}

void
AppWindow::about()
{
  about_dialog->exec();
}

AppWindow::AppWindow(OrderedSets *oss) : Gtk::Window(GTK_WINDOW_TOPLEVEL)
{
  Gtk::VBox *vbox;
  Gtk::MenuBar *menubar;
  AppFileMenu *filemenu;
  Gtk::Menu *helpmenu;
  string filemenuitem, helpmenuitem;
  
  orderedsets = oss;
  // TODO: will be in preferences
  copy_orderedset = new OrderedSet(OrderedSet::DATA_ARRAY);
  title = PROG_NAME;
  about_dialog = new AboutDialog;
  // set title
  set_title(title);
  // disable shrinking
  set_policy(false, true, false);
  destroy.connect(SigC::slot(*this, &AppWindow::destroy_handler));
  // menu
  menubar = SigC::manage(new Gtk::MenuBar);
  // shadow in
  menubar->set_shadow_type(GTK_SHADOW_ETCHED_IN);
  // file
  filemenu = SigC::manage(new AppFileMenu(this, oss));
  filemenuitem = _("_File");
  menubar->items().push_back(Gtk::Menu_Helpers::MenuElem
			     (filemenuitem, 
			      Gtk::Menu_Helpers::ALT | filemenuitem[1],
			      *filemenu));
  // help
  helpmenu = SigC::manage(new Gtk::Menu);
  helpmenuitem = _("_Help");
  menubar->items().push_back(Gtk::Menu_Helpers::MenuElem
			     (helpmenuitem, 
			      Gtk::Menu_Helpers::ALT | helpmenuitem[1],
			      *helpmenu));
  // tearoff
  helpmenu->items().push_back(Gtk::Menu_Helpers::TearoffMenuElem());
  // help
  /*
    helpmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
    (_("Help..."), "F1", SigC::slot(*this, &AppWindow::help)));
  */
  // about
  helpmenu->items().push_back
    (Gtk::Menu_Helpers::MenuElem
     (_("About..."), SigC::slot(*this, &AppWindow::about)));
  menubar->show();
  // buttons
  buttons = SigC::manage(new AppButtons);
  buttons->show();
  vbox = SigC::manage(new Gtk::VBox(false, 0));
  // menubar may not grow
  vbox->pack_start(*menubar, false, false, 0);
  // buttons may
  vbox->pack_start(*buttons, true, true, 0);
  vbox->show();
  add(*vbox);
  show();
}

AppWindow::~AppWindow()
{
  delete about_dialog;
}

void
AppWindow::close()
{
  if (reallyClose()) destroy();
}
