/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// class of the diagram area

#ifndef __DIAGRAMAREA_H__
#define __DIAGRAMAREA_H__

#include "orderedsets.h"
#include <qwidget.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <list>

class DiagramWindow;
class DiagramPopupMenu;
class SelectPathDialog;

class DiagramArea : public QWidget {
 protected:
  const int diagram_vertex_size;
  const int diagram_border_x;
  const int diagram_border_y;
  int min_diagram_width, min_diagram_height;
  QColor color, back_color, selected_color, marked_color;
  QFont text_font;
  QPixmap pixmap, back_pixmap;
  QPainter pixmap_painter, back_pixmap_painter;
  OrderedSets *orderedsets;
  OrderedSet *orderedset;
  DiagramWindow *window;
  DiagramPopupMenu *popupmenu;
  SelectPathDialog *selectpath_dialog;
  set<Element> selected_elements;
  set<Element> marked_elements;
  list<Element> mode_elements;
  list<set<Element> > path_elems;
  Vertex *drag_vertex;
  Coord_t drag_old_diagram_coord_x;
  Coord_t drag_old_diagram_coord_y;
  void coords_diagram_to_area(int &x, int &y, int min_x, int min_y,
                              int w, int h);
  void coords_area_to_diagram(int &x, int &y, int min_x, int min_y,
                              int w, int h);
  void drawVertex(QPainter *painter, int x, int y, string label, 
		  bool selected, bool marked);
  void drawDiagram();
  void flush(int x, int y, int w, int h);
  void resizeEvent(QResizeEvent *ev);
  void paintEvent(QPaintEvent *ev);
  void mousePressEvent(QMouseEvent *ev);
  void mouseMoveEvent(QMouseEvent *ev);
 public:
  DiagramArea(QWidget *parent, DiagramWindow *win, DiagramPopupMenu *pmenu,
	      OrderedSets *oss, OrderedSet *os, int min_width,
	      int min_height);
  ~DiagramArea() {}
  void update();
  set<Element> getSelectedElements() { return selected_elements; }
  void clearSelectedElements() { selected_elements.clear(); }
  void removeSelectedElements();
  void invertSelectedElements();
  void selectMarkedElements() { selected_elements = marked_elements; }
  void clearMarkedElements() { marked_elements.clear(); }
  void markElements(set<Element> s) { marked_elements = s; }
  void getMinDiagramSize(int &w, int &h) { 
    w = min_diagram_width; h = min_diagram_height;
  }
  void getDiagramBorder(int &x, int &y) { 
    x = diagram_border_x; y = diagram_border_y;
  }
  int getDiagramVertexSize() { return diagram_vertex_size; }
};

#endif
