/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "diagramwindow.h"
#include "diagramarea.h"
#include "diagrampopupmenu.h"
#include "dialogs.h"
#include "diagram_graphic.h"
#include "i18n.h"
#include <qscrollview.h>
#include <qlayout.h>
#include <stdio.h>

bool
DiagramWindow::reallyClose()
{
  QuestionDialog *question_dialog;
  bool answer = true;
  QString name;
  
  if (!orderedset->isFileSaved()) {
    name = title; name.remove(0, 1);
    question_dialog = 
      new QuestionDialog(name, 
			 QString::fromLocal8Bit(_("Changes were made to")) +
			 " " + name + ".", 
			 QString::fromLocal8Bit(_("Close anyway?")));
    answer = question_dialog->exec();
    delete question_dialog;
  }
  return answer;
}

void
DiagramWindow::destroy(bool destroyWindow, bool destroySubWindows)
{
  QWidget::destroy(destroyWindow, destroySubWindows);
  // TODO: causes segfault, why??!
  //orderedsets->unrefSet(orderedset);
}

void
DiagramWindow::closeEvent(QCloseEvent *ev)
{
  if (reallyClose()) QWidget::closeEvent(ev);
}

DiagramWindow::DiagramWindow(AppWindow *appwin, OrderedSets *oss,
			     OrderedSet *os, int min_width, int min_height)
  : QWidget()
{
  QScrollView *scrollview;
  QVBoxLayout *vbox_scrollview, *vbox_area;

  orderedsets = oss;
  orderedset = os;
  setTitle();
  // scrollview
  scrollview = new QScrollView(this);
  // automatic scrollbars
  scrollview->setVScrollBarMode(QScrollView::Auto);
  scrollview->setHScrollBarMode(QScrollView::Auto);
  scrollview->show();
  // viewport shadow
  popupmenu = new DiagramPopupMenu(appwin, this, oss, os);
  area = new DiagramArea(scrollview->viewport(), this, popupmenu, oss, os,
			 min_width, min_height);
  popupmenu->setArea(area);
  scrollview->addChild(area);
  vbox_area = new QVBoxLayout(scrollview->viewport());
  vbox_area->addWidget(area);
  vbox_scrollview = new QVBoxLayout(this);
  vbox_scrollview->addWidget(scrollview);
  show();
}

void
DiagramWindow::update()
{
  area->update();
}

void
DiagramWindow::setTitle()
{
  char not_saved_mark = '*';
  
  title = orderedset->getFilename().c_str();
  if (title == "") title = QString::fromLocal8Bit(_("Untitled"));
  else title = title.remove(0, title.findRev('/') + 1);
  if (!orderedset->isFileSaved()) {
    if (title[0] != not_saved_mark) {
      title.insert(0, not_saved_mark);
      setCaption(title);
    }
  }
  else setCaption(title);
}
