/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "orderdata_list.h"
#include <stdio.h>

bool
OrderData_List::isElement(ElementId x)
{
  ListItem item;
  
  item.element = x;
  if (list_upper.find(item) != list_upper.end()) return true;
  return false;
}

bool
OrderData_List::insertPair(ElementId x, ElementId y)
{
  ListItem item;
  set<ListItem>::iterator iter_s;
  
  item.element = x;
  iter_s = list_upper.find(item);
  if (iter_s != list_upper.end()) { 
    item = *iter_s;
    list_upper.erase(iter_s);
  }
  item.list.insert(y);
  list_upper.insert(item);
  item.element = y;
  iter_s = list_lower.find(item);
  if (iter_s != list_lower.end()) { 
    item = *iter_s;
    list_lower.erase(iter_s);
  }
  item.list.insert(x);
  list_lower.insert(item);
  return true;
}

bool
OrderData_List::erasePair(ElementId x, ElementId y)
{
  ListItem item;
  set<ListItem>::iterator iter_s;
  bool result;
  
  item.element = x;
  iter_s = list_upper.find(item);
  if (iter_s != list_upper.end()) { 
    item = *iter_s;
    list_upper.erase(iter_s);
  }
  else return false;
  result = (item.list.erase(y) != 0);
  list_upper.insert(item);
  if (!result) return false;
  item.element = y;
  iter_s = list_lower.find(item);
  if (iter_s != list_lower.end()) { 
    item = *iter_s;
    list_lower.erase(iter_s);
  }
  else return false;
  result = (item.list.erase(x) != 0);
  list_lower.insert(item);
  if (!result) return false;
  return true;
}

bool
OrderData_List::lessTransitive(ElementId x, ElementId y, bool transitivity)
{
  /*
    for all (x, z) do recursively for (z, y)
  */
  ListItem item;
  set<ListItem>::iterator iter_s;
  set<ElementId>::iterator iter_si;

  item.element = x;
  iter_s = list_upper.find(item);
  if (iter_s == list_upper.end()) return false;
  if ((*iter_s).list.find(y) != (*iter_s).list.end()) return true;
  if (!transitivity) return false;
  for (iter_si = (*iter_s).list.begin(); iter_si != (*iter_s).list.end();
       iter_si++)
    if (lessTransitive(*iter_si, y)) return true;
  return false;
}

void
OrderData_List::minimal_maximal(set<ElementId> &s, set<ElementId> ss, 
				bool minimal)
{
  /*
    minimal m: does not exist (x, m), x != m, x, m from ss
    maximal M: does not exist (M, x), M != x, M, x from ss
  */
  set<ElementId>::iterator iter_s1, iter_s2;
  
  for (iter_s1 = ss.begin(); iter_s1 != ss.end(); iter_s1++) {
    for (iter_s2 = ss.begin(); iter_s2 != ss.end(); iter_s2++) {
      if (*iter_s1 == *iter_s2) continue;
      if (minimal) { if (lessTransitive(*iter_s2, *iter_s1)) break; }
      else if (lessTransitive(*iter_s1, *iter_s2)) break;
    }
    if (iter_s2 == ss.end()) s.insert(*iter_s1);
  }
}

ElementId
OrderData_List::least_greatest(set<ElementId> ss, bool least)
{
  /*
    least m: for all x is (m, x), m, x from ss
    greatest M: for all x is (x, M), x, M from ss
  */
  set<ElementId>::iterator iter_s1, iter_s2;
  
  for (iter_s1 = ss.begin(); iter_s1 != ss.end(); iter_s1++) {
    for (iter_s2 = ss.begin(); iter_s2 != ss.end(); iter_s2++) {
      if (*iter_s1 == *iter_s2) continue;
      if (least) { if (!lessTransitive(*iter_s1, *iter_s2)) break; }
      else if (!lessTransitive(*iter_s2, *iter_s1)) break;
    }
    if (iter_s2 == ss.end()) return *iter_s1;
  }
  return NoElement;
}

void
OrderData_List::lower_upperCone(set<ElementId> &s, set<ElementId> ss, 
				bool lower)
{
  /*
    lower: all m such that for all x is (m, x), x from ss
    upper: all M such that for all x is (x, M), x from ss
  */
  set<ListItem>::iterator iter_s;
  set<ElementId>::iterator iter_se;
  
  for (iter_s = list_upper.begin(); iter_s != list_upper.end(); iter_s++) {
    for (iter_se = ss.begin(); iter_se != ss.end(); iter_se++) {
      if ((*iter_s).element == *iter_se) continue;
      if (lower) { 
	if (!lessTransitive((*iter_s).element, *iter_se)) break;
      }
      else if (!lessTransitive(*iter_se, (*iter_s).element)) break;
    }
    if (iter_se == ss.end()) s.insert((*iter_s).element);
  }
}

void
OrderData_List::getPaths_(list<set<ElementId> > &l, set<ElementId> s, 
			   ElementId x, ElementId y)
{
  ListItem item;
  set<ListItem>::iterator iter_s;
  set<ElementId>::iterator iter_se;
  bool transitive = false;
  pair<set<ElementId>::iterator, bool> p;
  
  item.element = x;
  iter_s = list_upper.find(item);
  if (iter_s == list_upper.end()) return;
  for (iter_se = (*iter_s).list.begin(); iter_se != (*iter_s).list.end();
       iter_se++) {
    if (!lessTransitive(*iter_se, y, true)) continue;
    transitive = true;
    p = s.insert(*iter_se);
    getPaths_(l, s, *iter_se, y);
    s.erase(p.first);
  }
  if (!transitive) l.push_back(s);
}

OrderData_List::~OrderData_List()
{
}

OrderData *
OrderData_List::getCopy()
{
  OrderData_List *copy;

  copy = new OrderData_List();
  if (!copy) return NULL;
  copy->list_upper = list_upper;
  copy->list_lower = list_lower;
  return copy;
}

void
OrderData_List::clear()
{
  list_upper.clear();
  list_lower.clear();
}

bool
OrderData_List::newElement(ElementId x)
{
  ListItem item;
  
  item.element = x;
  if (!list_upper.insert(item).second) return false;
  list_lower.insert(item);
  return true;
}

bool
OrderData_List::removeElement(ElementId x)
{
  /*
    remove also all pairs containing x
  */
  ListItem item;
  set<ListItem>::iterator iter_s1, iter_s2;
  set<ElementId>::iterator iter_siu, iter_sil;
  set<ElementId> list_copy;

  item.element = x;
  iter_s1 = list_upper.find(item);
  if (iter_s1 == list_upper.end()) return false;
  list_copy = (*iter_s1).list;
  list_upper.erase(iter_s1);
  for (iter_siu = list_copy.begin(); iter_siu != list_copy.end(); 
       iter_siu++) {
    item.element = *iter_siu;
    iter_s2 = list_lower.find(item);
    item = *iter_s2;
    list_lower.erase(iter_s2);
    item.list.erase(x);
    list_lower.insert(item);
    item.element = x;
    iter_s2 = list_lower.find(item);
    for (iter_sil = (*iter_s2).list.begin(); 
	 iter_sil != (*iter_s2).list.end(); iter_sil++)
      if (!(lessTransitive(*iter_siu, *iter_sil) || 
	    lessTransitive(*iter_sil, *iter_siu)))
	insertPair(*iter_sil, *iter_siu);
  }
  item.element = x;
  iter_s1 = list_lower.find(item);
  for (iter_sil = (*iter_s1).list.begin(); iter_sil != (*iter_s1).list.end();
       iter_sil++) {
    item.element = *iter_sil;
    iter_s2 = list_upper.find(item);
    item = *iter_s2;
    list_upper.erase(iter_s2);
    item.list.erase(x);
    list_upper.insert(item);
  }
  list_lower.erase(iter_s1);
  return true;
}

bool
OrderData_List::orderElements(ElementId x, ElementId y)
{
  ListItem item;
  set<ListItem>::iterator iter_s;
  set<ElementId>::iterator iter_si;

  if ((!isElement(x)) || (!isElement(y))) return false;
  if (x == y) return false;
  if (lessTransitive(y, x) || lessTransitive(x, y)) return false;
  insertPair(x, y);
  item.element = y;
  iter_s = list_upper.find(item);
  for (iter_si = (*iter_s).list.begin(); iter_si != (*iter_s).list.end();
       iter_si++)
    erasePair(x, *iter_si);
  item.element = x;
  iter_s = list_lower.find(item);
  for (iter_si = (*iter_s).list.begin(); iter_si != (*iter_s).list.end();
       iter_si++)
    erasePair(*iter_si, y);
  return true;
}

bool
OrderData_List::removeOrder(ElementId x, ElementId y)
{
  ListItem item;
  set<ListItem>::iterator iter_s;
  set<ElementId>::iterator iter_si;

  if (x == y) return false;
  if (!erasePair(x, y)) return false;
  item.element = y;
  iter_s = list_upper.find(item);
  for (iter_si = (*iter_s).list.begin(); iter_si != (*iter_s).list.end();
       iter_si++)
    if (!(lessTransitive(*iter_si, x) || lessTransitive(x, *iter_si)))
      insertPair(x, *iter_si);
  item.element = x;
  iter_s = list_lower.find(item);
  for (iter_si = (*iter_s).list.begin(); iter_si != (*iter_s).list.end();
       iter_si++)
    if (!(lessTransitive(y, *iter_si) || lessTransitive(*iter_si, y)))
      insertPair(*iter_si, y);
  return true;
}

bool
OrderData_List::less(ElementId x, ElementId y, bool transitivity)
{
  if ((!isElement(x)) || (!isElement(y))) return false;
  if (x == y) return true;
  return lessTransitive(x, y, transitivity);
}

void
OrderData_List::getElements(set<ElementId> &s)
{
  set<ListItem>::iterator iter_s;
  
  for (iter_s = list_upper.begin(); iter_s != list_upper.end(); iter_s++)
    s.insert((*iter_s).element);
}

void
OrderData_List::upperElements(set<ElementId> &s, ElementId x)
{
  ListItem item;
  set<ListItem>::iterator iter_s;

  item.element = x;
  iter_s = list_upper.find(item);
  if (iter_s == list_upper.end()) return;
  s = (*iter_s).list;
}

void
OrderData_List::lowerElements(set<ElementId> &s, ElementId x)
{
  ListItem item;
  set<ListItem>::iterator iter_s;
  
  item.element = x;
  iter_s = list_lower.find(item);
  if (iter_s == list_lower.end()) return;
  s = (*iter_s).list;
}

void
OrderData_List::minimalElements(set<ElementId> &s, set<ElementId> ss)
{
  minimal_maximal(s, ss, true);
}

void
OrderData_List::maximalElements(set<ElementId> &s, set<ElementId> ss)
{
  minimal_maximal(s, ss, false);
}

ElementId
OrderData_List::leastElement(set<ElementId> ss)
{
  return least_greatest(ss, true);
}

ElementId
OrderData_List::greatestElement(set<ElementId> ss)
{
  return least_greatest(ss, false);
}

void
OrderData_List::lowerCone(set<ElementId> &s, set<ElementId> ss)
{
  lower_upperCone(s, ss, true);
}

void
OrderData_List::upperCone(set<ElementId> &s, set<ElementId> ss)
{
  lower_upperCone(s, ss, false);
}

ElementId
OrderData_List::infimum(set<ElementId> ss)
{
  set<ElementId> s1;
  
  lower_upperCone(s1, ss, true);
  return least_greatest(s1, false);
}

ElementId
OrderData_List::supremum(set<ElementId> ss)
{
  set<ElementId> s1;

  lower_upperCone(s1, ss, false);
  return least_greatest(s1, true);
}

void
OrderData_List::getPaths(list<set<ElementId> > &l, ElementId x,
			  ElementId y)
{
  set<ElementId> s;

  if (!lessTransitive(x, y, true)) return;
  s.insert(x); s.insert(y);
  getPaths_(l, s, x, y);
}
